@@ -3,17 +3,15 @@ ws.onopen = function() {
    ws.send("Hello");
 };
 
-window.event_times = [];
-window.parameter_types = [];
+window.limit = 20; // maximum number of kept entries for a parameter
 window.graphs = [];
-window.current_event = 0;
+window.parameter_types = [];
+window.parameters = new Object();
 
 ws.onmessage = function (evt) {
     var json_obj = JSON.parse(evt.data);
 
     if(json_obj.kind == "parameter") {
-    	var event_number = json_obj.event_number;
-        update_event_number(event_number);
     	process_parameter(json_obj);
     }
 
@@ -42,84 +40,204 @@ function log_message(data) {
     }
 }
 
+
 function clean_up_history(lines_to_keep) {
     if($("#console_log").children().length > 500) {
         $('#console_log').find('div').first().remove();
     }
 }
 
-function update_event_number(event_number) {
-    if(event_number > window.current_event) {
-    	var event_rate = calculate_event_rate();
-        window.current_event = event_number;
-    }
-}
 
-function calculate_event_rate() {
-    if(window.event_times.length > 10) {
-        window.event_times.shift();
-    }
-    var event_time = new Date().getTime();
-    window.event_times.push(event_time);
-    var latest = window.event_times.slice(-1)[0];
-    var first = window.event_times[0];
-    var count = window.event_times.length;
-    var event_rate = ((count - 1) / (latest - first) * 1000).toFixed(2);
-
-    d3.select("#event_rate").text(event_rate + " Hz");
-
-    if(event_rate > 2) {
-        $('#event_rate').css("color", "#FF5179");
-        $('#event_rate_indicator').css("background-color", "#FF5179");
-    } else {
-        $('#event_rate').css("color", "#268BD3");
-        $('#event_rate_indicator').css("background-color", "#268BD3");
-    }
-    $('#event_rate').stop(true, true).show();
-    $('#event_rate_indicator')
-        .stop(true, true)
-        .fadeIn(50)
-        .fadeOut(10, function(){
-            $('#event_rate').fadeOut(5000);
-        });
-
-    return event_rate;
-}
-
-function includes(arr,obj) {
+function includes(arr, obj) {
     return (arr.indexOf(obj) != -1);
 }
 
+function timestamp() {
+    // Return unix timestamp with milliseconds accuracy.
+    return +new Date()
+}
 
-function process_parameter(obj) {
-    if(!includes(window.parameter_types, obj.type)) {
-        window.parameter_types.push(obj.type);
-        d3.select("#parameter_types").append("div")
-                .attr("id", obj.type + "_tag")
-                .attr("class", "parameter_tag")
-                .attr("rel", obj.type)
-                .text(obj.type);
-        $("#"+obj.type + "_tag").click(function() {
-            $("#" + obj.type + "_graph").parent().fadeToggle();
-            $(this).toggleClass("inactive_parameter", 200);
-        }); 
+var guid = (function() {
+  function s4() {
+    return Math.floor((1 + Math.random()) * 0x10000)
+               .toString(16)
+               .substring(1);
+  }
+  return function() {
+    return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
+           s4() + '-' + s4() + s4() + s4();
+  };
+})();
 
-        var graph = new Graph(obj.type);
-        window.graphs.push(graph);
-
-        log_message("New parameter type received: " + obj.type);
+function process_parameter(parameter) {
+    if(!includes(window.parameter_types, parameter.type)) {
+        log_message("New parameter type received: " + parameter.type);
+        window.parameter_types.push(parameter.type);
+        var table_row = d3.select("#parameters").append("tr");
+        table_row.append("td")
+            .append("input")
+            .attr("type", "submit")
+            .attr("value", parameter.type)
+            .attr("class", "parameter_type")
+            .text(parameter.type);
+        table_row.append("td")
+            .attr("class", "parameter_rate")
+            .attr("id", parameter.type + "_rate")
+            .text("0");
+        window.parameters[parameter.type] = [];
+        var graph = new Graph();
+        graph.parameter_types.push(parameter.type);
+        graph.set_title(parameter.type);
     }
 
-    for (index = 0; index < window.graphs.length; ++index) {
-        graph = window.graphs[index];
-        if(graph.type == obj.type) {
-            graph.add_obj(obj);
+    if(window.parameters[parameter.type].length >= window.limit) {
+            window.parameters[parameter.type].shift();
+    }
+    window.parameters[parameter.type].push(parameter);
+
+    update_parameter_rate(parameter);
+    update_graphs(parameter);
+}
+
+function update_parameter_rate(parameter) {
+    var rate = calculate_parameter_rate(parameter);
+    d3.select("#" + parameter.type + "_rate").text(rate);
+}
+
+function calculate_parameter_rate(parameter) {
+    var parameters = window.parameters[parameter.type];
+    var latest = parameters.slice(-1)[0].time;
+    var first = parameters[0].time;
+    var count = parameters.length;
+    parameter_rate = ((count - 1) / (latest - first) * 1000).toFixed(2);
+    return parameter_rate;
+}
+
+function update_graphs(parameter) {
+    for(var index in window.graphs) {
+        var graph = window.graphs[index];
+        if(includes(graph.parameter_types, parameter.type)) {
             graph.redraw();
         }
     }
 }
 
-function Graph (parameter_type) {
+
+function Graph() {
+    // A graph, which automatically adds itself to the content-DIV
+    var self = {};
+
+    self.id = guid();
+
+    self.parameter_types = []; // TODO: multi-graph
+
+    self.w = 450;
+    self.h = 200;
+    self.padding = 25;
+    self.padding_left = 50;
+
+    self.div = d3.select("#content").append("div").attr("class", "graph");
+    self.title_field = self.div.append("h2");
+    self.svg = self.div.append("svg")
+                       .attr("width", self.w)
+                       .attr("height", self.h)
+                       .attr("id", self.id);
+
+    self.points = self.svg.append("g")
+        .attr("class", "points");
+
+
+    //self.xScale = d3.scale.linear().range([self.padding_left, self.w - self.padding]);
+    self.xScale = d3.time.scale().range([self.padding_left, self.w - self.padding]);
+    self.yScale = d3.scale.linear().range([self.h - self.padding, self.padding]);
+
+
+    var formatAsPercentage = d3.format(".1%");
+    var timeFormat = d3.time.format("%X");
+
+    self.xAxis = d3.svg.axis().scale(self.xScale).orient("bottom").ticks(5)
+                              .tickPadding(5)
+                              .tickFormat(timeFormat);
+    self.yAxis = d3.svg.axis().scale(self.yScale).orient("left").ticks(5);
+
+    self.svg.append("g")
+        .attr("class", "x axis")
+        .attr("transform", "translate(0," + (self.h - self.padding) + ")")
+        .call(self.xAxis);
+    self.svg.append("g")
+        .attr("class", "y axis")
+        .attr("transform", "translate(" + self.padding_left + ",0)")
+        .call(self.yAxis);
+
+
+
+    self.set_title = function(title) {
+        // Update the H2 field of the graph.
+        self.title_field.text(title);
+    }
+
+    self.resize = function(width, height) {
+        // Change the width and height of the SVG container.
+        self.svg.attr("width", width).attr("height", height);
+    }
+
+    self.redraw = function() {
+        for(var index in self.parameter_types) {
+            //console.log(index);
+        }
+        var data = window.parameters[self.parameter_types[0]]; // TODO: multi-graphs
+
+        self.xScale.domain([
+                           d3.min(data, function(d) { return d.time; }),
+                           d3.max(data, function(d) { return d.time; })
+                           ]);
+
+        self.yScale.domain([
+                           d3.min(data, function(d) { return d.value; }),
+                           d3.max(data, function(d) { return d.value; })
+                           ]);
+
+        self.svg.select(".x.axis")
+            .transition()
+            .duration(500)
+            .call(self.xAxis)
+        self.svg.select(".y.axis")
+            .transition()
+            .duration(500)
+            .call(self.yAxis)
+
+        var points = self.points.selectAll("circle")
+                      .data(data, function(d) { return d.time; })
+
+
+        points.enter()
+              .append("circle")
+              .attr("cx", function(d) {
+                  return self.xScale(d.time);
+              })
+              .attr("cy", function(d) {
+                  return self.yScale(d.value);
+              })
+              .attr("r", 2)
+              .attr("fill", "#268BD3");
+
+        points.transition()
+              .duration(500)
+              .attr("cx", function(d) {
+                  return self.xScale(d.time);
+              });
+
+        points.exit()
+              .transition()
+              .duration(500)
+              .remove();
+    }
+
+    window.graphs.push(self);
+    return self;
+}
+
+function Graph_ (parameter_type) {
     var self = {};
 
     self.dataset = [];
